import React, {useContext, useMemo} from "react";
import {ColumnContext, ColumnKey} from "./Column";
import {CellDefinition, CellInstance} from "./Cell";
import {Smartable, useTable} from "./Smartable";
import {Async, Promisable} from "@kernelui/core";

/**
 * Smartable row cells.
 */
export type RowCells<CK extends ColumnKey, T = any> = Record<CK, Promisable<CellDefinition<T>>>;

/**
 * Smartable row data.
 */
export interface RowData<CK extends ColumnKey, T = any>
{
	/**
	 * Cells definition.
	 */
	cells: RowCells<CK, T>;

	/**
	 * Rendered row element.
	 */
	element?: React.ReactElement;

	/**
	 * Rendered row cell element.
	 */
	cellElement?: React.ReactElement;
}

/**
 * Smartable row definition.
 */
export type RowDefinition<CK extends ColumnKey> = RowCells<CK>|RowData<CK>;

/**
 * Table row context data.
 */
export interface RowContextData<CK extends ColumnKey, T = any> extends RowData<CK, T>
{
}

const RowContext = React.createContext<RowContextData<ColumnKey>>(undefined);

/**
 * Hook to get current row data.
 */
export function useRow<CK extends ColumnKey, T = any>(smartable?: Smartable<CK>): RowContextData<CK, T>
{
	return useContext(RowContext);
}

/**
 * Default row component.
 */
export function Row()
{
	return (
		<tr>
			<RowCells />
		</tr>
	);
}

export function RowCells()
{
	// Get row data.
	const row = useRow();

	// Get table data.
	const {columns} = useTable();

	return (
		Object.entries(columns).map(([columnKey, column]) => (
			<ColumnContext.Provider key={columnKey} value={{ key: columnKey, column: column }}>
				<Async<CellDefinition> promise={row.cells?.[columnKey] ?? { data: undefined }}>
					{(cellDefinition) => (
						<CellInstance cell={cellDefinition} />
					)}
				</Async>
			</ColumnContext.Provider>
		))
	);
}

/**
 * Row instance component.
 */
export function RowInstance<CK extends ColumnKey>({row}: { row: RowDefinition<CK> })
{
	// Get row context value from given row definition.
	const rowContextValue = useMemo(() => (
		// If a simple RowCells<CK> object is given, converting it to a RowData<CK>
		!("cells" in row) ? { cells: row } : row
	), [row]);

	const {rowElement} = useTable();

	return (
		<RowContext.Provider value={rowContextValue}>
			{ // Trying to render row-specific element, then table-specific element, then default element.
				rowContextValue.element ?? rowElement ?? <Row />
			}
		</RowContext.Provider>
	);
}
